from metrics import config
from metrics.util import get_mesos_slaves
from requests import get
from collections import defaultdict

def get_slaves_attr(slaves_state):
    attrs = defaultdict(set)

    for slave in slaves_state['slaves']:
        for attr in slave['attributes']:
            attrs[attr].add(slave['attributes'][attr])

    for attr in attrs:
        attrs[attr] = list(attrs[attr])

    return dict(attrs)

def get_slaves_with_attr(slaves_state, attrs):
    slaves = []

    for slave in slaves_state['slaves']:
        intersection = dict(set(slave['attributes'].items()).intersection(set(attrs.items())))

        if intersection != attrs:
            continue

        slaves.append(slave)

    return slaves

def get_attr_usage(slaves_state, attrs):
    slaves = get_slaves_with_attr(slaves_state, attrs)

    usage = 0

    cpu_total = 0
    ram_total = 0

    cpu_used = 0
    ram_used = 0

    for slave in slaves:
        cpu_total = cpu_total + slave["resources"]['cpus']
        ram_total = ram_total + slave["resources"]['mem']

        cpu_used = cpu_used + slave["used_resources"]['cpus']
        ram_used = ram_used + slave["used_resources"]['mem']

    ram_total = round(ram_total/1000)
    ram_used = round(ram_used/1000)

    return {
        'cpu_total': cpu_total,
        'ram_total': ram_total,
        'cpu_used': cpu_used,
        'ram_used': ram_used,
        'cpu_pct': round(cpu_used*100/cpu_total, 1),
        'ram_pct': round(ram_used*100/ram_total, 1)
    }




import requests


from metrics import config


def get_mesos_leader_address():
    for mesos_address in config.MESOS_ADDRESSES:
        try:
            response = requests.get(f"{mesos_address}/redirect", timeout=2, allow_redirects=False)
            if response.headers.get("Location"):
                leader_ip = response.headers.get("Location").split("//")[1]
                config.logger.debug({"action": "find-mesos-leader", "try-address": mesos_address, "exception": False, "leader": True})
                return f"http://{leader_ip}"
        except requests.exceptions.ConnectionError as ConErr:
            config.logger.debug({"action": "find-mesos-leader", "try-address": mesos_address, "exception": True})

def get_mesos_slaves():
    leader_address = get_mesos_leader_address()
    url = f"{leader_address}/slaves"
    config.logger.debug({"action": "pre-fetch", "fetch-url": url})
    response = requests.get(url, timeout=2)
    config.logger.debug({"action": "post-fetch", "fetch-url": url, "fetch-status": response.status_code})
    return response.json()



from json import dumps
import json

import requests
from flask import Blueprint, Response, request

from metrics import mesos, config
from metrics.util import get_mesos_leader_address

mesos_metrics_blueprint = Blueprint(__name__, __name__)

def asgard_api_plugin_init(**kwargs):
    config.logger = kwargs.get("logger", config.logger)
    return {
        'blueprint': mesos_metrics_blueprint
    }

@mesos_metrics_blueprint.route('/attrs/count')
def attrs_count():
    config.logger.debug("Counting attrs")
    slaves_state = mesos.get_mesos_slaves()
    slaves_attrs = mesos.get_slaves_attr(slaves_state)
    return Response(
        dumps({'total_attrs': len(slaves_attrs.keys())}),
        mimetype='application/json'
    )

@mesos_metrics_blueprint.route('/attrs')
def attrs():
    config.logger.debug("Reading attrs")
    slaves_state = mesos.get_mesos_slaves()
    return Response(
        dumps(mesos.get_slaves_attr(slaves_state)),
        mimetype='application/json'
    )


@mesos_metrics_blueprint.route('/slaves-with-attrs/count')
def slaves_with_attrs_count():
    slaves_state = mesos.get_mesos_slaves()
    result = mesos.get_slaves_with_attr(
        slaves_state,
        dict(request.args.items())
    )
    return Response(
        dumps({'total_slaves': len(result)}),
        mimetype='application/json'
    )

@mesos_metrics_blueprint.route('/slaves-with-attrs')
def slaves_with_attrs():
    slaves_state = mesos.get_mesos_slaves()
    result = mesos.get_slaves_with_attr(
        slaves_state,
        dict(request.args.items())
    )
    return Response(
        dumps(result),
        mimetype='application/json'
    )

@mesos_metrics_blueprint.route('/attr-usage')
def slaves_attr_usage():
    slaves_state = mesos.get_mesos_slaves()
    result = mesos.get_attr_usage(
        slaves_state,
        dict(request.args.items())
    )
    return Response(
        dumps(result),
        mimetype='application/json'
    )


def filter_mesos_metrics(server_address, prefix):
    metrics_data = requests.get(f"{server_address}/metrics/snapshot").json()
    filtered_metrics = {}
    for data_key, data_value in metrics_data.items():
        if data_key.startswith(prefix):
            filtered_metrics[data_key] = data_value
    return filtered_metrics

@mesos_metrics_blueprint.route("/leader")
def leader_metrics():
    prefix = request.args.get("prefix", "")
    server_address = get_mesos_leader_address()
    filtered_metrics = filter_mesos_metrics(server_address, prefix)
    return Response(json.dumps(filtered_metrics), mimetype='application/json')

@mesos_metrics_blueprint.route("/master/<string:server_ip>")
def master_metrics(server_ip):
    prefix = request.args.get("prefix", "")
    filtered_metrics = filter_mesos_metrics(f"http://{server_ip}:5050", prefix)
    return Response(json.dumps(filtered_metrics), mimetype='application/json')

@mesos_metrics_blueprint.route("/slave/<string:server_ip>")
def slave_metrics(server_ip):
    prefix = request.args.get("prefix", "")
    filtered_metrics = filter_mesos_metrics(f"http://{server_ip}:5051", prefix)
    return Response(json.dumps(filtered_metrics), mimetype='application/json')




from setuptools import setup, find_packages  # Always prefer setuptools over distutils
from codecs import open  # To use a consistent encoding
from os import path

here = path.abspath(path.dirname(__file__))

setup(
    name='asgard-api-plugin-metrics-mesos',
    version='0.5.0-rc2',

    description='Asgard API endpoints to get Apache Mesos metrics',
    long_description="Plugin para a Asgard API e que fornece métricas do cluster de Apache Mesos",
    url='https://github.com/B2W-BIT/asgard-api-plugin-metrics-mesos',
    # Author details
    author='Dalton Barreto',
    author_email='daltonmatos@gmail.com',
    license='MIT',
    classifiers=[
        'Programming Language :: Python :: 3.6',
    ],

    packages=find_packages(exclude=['contrib', 'docs', 'tests*']),

    entry_points={
        'asgard_api_metrics_mountpoint': [
            'init = metrics:asgard_api_plugin_init',
        ],
    },
)
